@@ -26,7 +26,7 @@ __version__   = "$Revision$"
 __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc." \
                 "Copyright (c) 2010-2011 The Orca Team" \
-                "Copyright (c) 2012 Igalia"
+                "Copyright (c) 2012 Igalia, S.L."
 __license__   = "LGPL"
 
 import argparse
@@ -61,6 +61,9 @@ try:
     #
     from gi.repository import Gtk
     from gi.repository import Gdk
+    # Note: This last import is here due to bgo #673396.
+    # See bgo#673397 for the rest of the story.
+    from gi.repository.GdkX11 import X11Screen
 except:
     pass
 
@@ -507,10 +510,10 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
             debug.println(debug.LEVEL_FINE,
                           "LOCUS OF FOCUS: None event=None")
     else:
-        if not app:
-            appname = "None"
-        else:
+        try:
             appname = "'" + app.name + "'"
+        except:
+            appname = "None"
         debug.println(debug.LEVEL_FINE,
                       "LOCUS OF FOCUS: app=%s name='%s' role='%s'" \
                       % (appname,
@@ -579,6 +582,8 @@ def _processKeyboardEvent(event):
         if isPressedEvent:
             script.presentationInterrupt()
         keyboardEvent.present()
+        if keyboardEvent.isModifierKey() and not isOrcaModifier:
+            return False
  
     # Special modes.
     if not isPressedEvent and keyboardEvent.event_string == "Escape":
@@ -733,6 +738,8 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     Returns True to indicate the input event has been consumed.
     """
 
+    debug.println(debug.LEVEL_FINEST, 'INFO: Loading User Settings')
+
     global _userSettings
 
     # Shutdown the output drivers and give them a chance to die.
@@ -838,6 +845,8 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
         dbusserver.init()
     httpserver.init()
 
+    debug.println(debug.LEVEL_FINEST, 'INFO: User Settings Loaded')
+
     return True
 
 def showAppPreferencesGUI(script=None, inputEvent=None):
@@ -1190,6 +1199,8 @@ def init(registry):
     module has already been initialized.
     """
 
+    debug.println(debug.LEVEL_FINEST, 'INFO: Initializing Orca module')
+
     global _initialized
 
     if _initialized and _settingsManager.isScreenReaderServiceEnabled():
@@ -1213,12 +1224,17 @@ def init(registry):
     # so that we respond when gnome-control-center is used to stop Orca.
     if a11yAppSettings:
         a11yAppSettings.connect('changed', onEnabledChanged)
+
+    debug.println(debug.LEVEL_FINEST, 'INFO: Orca module initialized')
+
     return True
 
 def start(registry):
     """Starts Orca.
     """
 
+    debug.println(debug.LEVEL_FINEST, 'INFO: Starting Orca')
+
     if not _initialized:
         init(registry)
 
@@ -1236,6 +1252,7 @@ def start(registry):
     if settings.cacheValues:
         pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
+    debug.println(debug.LEVEL_FINEST, 'INFO: Orca starting registry')
     registry.start(gil=settings.useGILIdleHandler)
 
 def die(exitCode=1):
@@ -1254,6 +1271,7 @@ def timeout(signum=None, frame=None):
     debug.println(debug.LEVEL_SEVERE,
                   "TIMEOUT: something has hung.  Aborting.")
     debug.printStack(debug.LEVEL_ALL)
+    examineProcesses()
     die(EXIT_CODE_HANG)
 
 def shutdown(script=None, inputEvent=None):
@@ -1263,6 +1281,8 @@ def shutdown(script=None, inputEvent=None):
     was never initialized.
     """
 
+    debug.println(debug.LEVEL_FINEST, 'INFO: Shutting down Orca')
+
     global _initialized
 
     if not _initialized:
@@ -1294,8 +1314,11 @@ def shutdown(script=None, inputEvent=None):
     _initialized = False
     _restoreXmodmap(_orcaModifiers)
 
+    debug.println(debug.LEVEL_FINEST, 'INFO: Orca stopping registry')
     pyatspi.Registry.stop()
 
+    debug.println(debug.LEVEL_FINEST, 'INFO: Orca shutdown complete')
+
     return True
 
 exitCount = 0
@@ -1348,6 +1371,34 @@ def abortOnSignal(signum, frame):
                   % signum)
     die(signum)
 
+def getCmdline(pid):
+    try:
+        openFile = os.popen('cat /proc/%s/cmdline' % pid)
+        cmdline = openFile.read()
+        openFile.close()
+    except:
+        cmdline = '(Could not obtain cmdline)'
+    cmdline = cmdline.replace('\x00', ' ')
+
+    return cmdline
+
+def examineProcesses():
+    desktop = pyatspi.Registry.getDesktop(0)
+    debug.println(
+        debug.LEVEL_ALL, 'INFO: Desktop has %i apps:' % desktop.childCount)
+    for i, app in enumerate(desktop):
+        pid = app.get_process_id()
+        cmd = getCmdline(pid)
+        try:
+            name = app.name
+        except:
+            name = 'ERROR: Could not get name'
+        else:
+            if name == '':
+                name = 'WARNING: Possible hang'
+        debug.println(
+            debug.LEVEL_ALL, '%3i. %s (pid: %s) %s' % (i+1, name, pid, cmd))
+
 def multipleOrcas():
     """Returns True if multiple instances of Orca are running
     which are owned by the same user."""
